// PJRC-MP3-CFastDL.cpp
// Fast firmware upload for PJRC MP3 players using updated 87c52 code.
// Based on xmit115200.c from Paul
// Visual C++ version by Zachary Bedell <aramis@adirondack.net>
// Testing w/ VC.Net on Windows XP.  Other platforms should work, but no guarantees
// Released under GNU GPL.
//
// To build:  Just drop this file into a Win32 Console project & compile.
// Usage: Take your pick -- it works via stdin or reading a file
//  xmit115200_VC COM1 mp3player.fdl
//  xmit115200_VC COM1 < mp3player.fdl
//  hex2fdl | xmit115200_VC COM1
//  xmit115200_VC < mp3player.fdl
//  hex2fdl | xmit115200_VC
//
// Defaults to COM1 if not specified
//
// WARNING:  This will do a Z Flash erase of your player before starting.  If you
// specify garbage for the command line, you'll get a blank player for your troubles.
// Not a big deal -- just reflash w/ something good, but be warned...

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <stdio.h>
#include "windows.h"



int main(int argc, char *argv[]) {
	// variables used with the com port	
	HANDLE			hCom, hConsole;	
	DCB				dcb;	
	DWORD			iBytesWritten, iBytesRead;
	LPVOID			lpMsgBuf;  // Holds error messages from FormatMessage
	unsigned char	buf[256];
	unsigned char	baud_change_cmd[2] = {0x6A, 0xB4};
	long int		i = 0;
	BOOL			do115200 = TRUE;


	printf(	"\nPJRC MP3 Player Fast Firmware Upload (Windows beta 1)\n"
		"Requires Rev. C or modified Rev A/B board & > May 11, 2002 87C52 code.\n"
		"   Opening COM port at 19200 bps...\n"
	);
	fflush(stdout);

	// if ARGV[1] is present, assume it's a COM port
	if(argc > 1) {
		hCom = CreateFile(argv[1], GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);	
	} else {
		hCom = CreateFile("COM1", GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);	
	}
	
	if(hCom == INVALID_HANDLE_VALUE) {	
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf, 0, NULL 
		);
		printf("Open port failed: %s\n", (LPCTSTR)lpMsgBuf);				
		LocalFree(lpMsgBuf);			
		return 1;
	}

	// If ARGV[2] is present, assume it's a file to upload (instead of stdin)
	if(argc > 2) {
		hConsole = CreateFile(argv[2], GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	} else {
		hConsole = GetStdHandle(STD_INPUT_HANDLE);
	}
	
	if(hConsole == INVALID_HANDLE_VALUE) {	
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf, 0, NULL 
		);
		printf("Open FDL file failed: %s\n", (LPCTSTR)lpMsgBuf);				
		LocalFree(lpMsgBuf);			
		return 1;
	}
	
	// Setup the COM port for 19200
	SetupComm(hCom, 256, sizeof(buf) * 2); // set buffer sizes	
	GetCommState(hCom, &dcb);
	dcb.BaudRate = 19200;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fAbortOnError = TRUE;
	SetCommState(hCom, &dcb);

	// Send erase flash & jump to 115200
	printf("   Erasing Flash (Z)...\n"); fflush(stdout);
	WriteFile(hCom, "Z",1,&iBytesWritten,NULL);
	Sleep(100);
	WriteFile(hCom, "Y",1,&iBytesWritten,NULL);
	Sleep(100);

	printf("   Setting player to Fast Flash mode...\n"); fflush(stdout);
	WriteFile(hCom, "~",1,&iBytesWritten,NULL);
	Sleep(1400);

	if(do115200) {	
		// Send speed up command to player & wait
		printf("   Setting player to 115200 bps\n"); fflush(stdout);
		WriteFile(hCom, baud_change_cmd,2,&iBytesWritten,NULL);	
		Sleep(200);
		
		// Set port to 115200
		printf("   Setting COM port to 115200 bps\n"); fflush(stdout);		
		dcb.BaudRate = 115200;
		SetCommState(hCom, &dcb);
	} // end if do 115200
	

	printf("   Starting Flash Upload...\n"); fflush(stdout);
	while (1) {
		printf("   Writing: ");
		//fflush(stdout);
		if(! ReadFile(hConsole, buf, sizeof(buf), &iBytesRead, 0)) {							
			FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf, 0, NULL 
			);
			printf("Read from input failed: %s\n", (LPCTSTR)lpMsgBuf);				
			LocalFree(lpMsgBuf);			
			return 1;
		}		
		if (iBytesRead == 0) break;		
		i += iBytesRead;		
		
		WriteFile(hCom, buf, iBytesRead, &iBytesWritten, NULL);
			
		printf("%d Bytes\r", i);
		fflush(stdout);
	}
	
	CloseHandle(hConsole);	// Should we not do this if it was a console instead of a file?
	CloseHandle(hCom);
	printf("\nDone!\n\n"); 
	fflush(stdout);
	return 0;
}

